==> LL_utils-4.1.1.2-x86_64-Linux-RHEL6.tar.gz: var/www/ll/cgi-bin/editable_table_insert.pl <==
#!/usr/bin/perl

# static char sccsid[] = "@(#)14 1.9 src/ll/config_editor/cgi-bin/editable_table_insert.pl, ll.config, ll_rrur2, rrur21042a 1/12/10 03:38:13\n";

use strict;
use lib "/var/www/ll/cgi-bin";
use LLUtil;
use DBI;

# table=TLLR_CFGMachineClass&nodeID=2&class_name=c&class_count=4&type=all_value&name=default&class_name=a&class_count=2&class_name=b&class_count=3
# table=TLLR_CFGMachineClass&nodeID=21&class_name=g&class_count=8&type=all_value&name=faaffa
my $query = $ENV{QUERY_STRING};
$query = unescape($query);

# connect to database
my $dbh = get_dbh(0);
if(!defined($dbh))
{
	error_and_exit(DBI->errstr);
}

# beginning of process input string

my @pairs = split('&',$query);
my @tmp = split('=',$pairs[0]);
my $table = $tmp[1];
@tmp = split('=',$pairs[1]);
my $current_id = $tmp[1];

# format the insert SQL string
my $insert_str="insert into ".$table." set";
my $count = 1;
my $n = scalar(@pairs);
for(my $i=1;$i<$n;$i++)
{
	if($pairs[$i] eq "type=all_value")
	{
		last;
	}
	
	@tmp = split('=',$pairs[$i]);
	$insert_str .= " ".$tmp[0]."='".$tmp[1]."',";
	$count++;
}
$insert_str =~ s/,$//g;
debug_log($insert_str);
debug_log("count: ".$count);

# data checking (make use of check utility code)
{
	# get stanza name
	my $index = $count + 1;
	@tmp = split('=',$pairs[$index]);
	my $stanza_name = $tmp[1];

	# format the <keyword>,<value> pair for checking
	my $keyword_string = "";
	if($index + 1 < $n)
	{
		my $column_number = $count - 2;
		my $existed_rows = ($n - $index - 1) / $column_number;
		debug_log("log: n=".$n.", index=".$index.", column_number=".$column_number.", existed_rows=".$existed_rows);


		for(my $i=0;$i<$existed_rows;$i++)
		{
			my @columns = ();
			push(@columns,$current_id);
			for(my $j=0;$j<$column_number;$j++)
			{
				@tmp = split('=',$pairs[$index + 1 + $i * $column_number + $j]);
				push(@columns,$tmp[1]);				
			}
			$keyword_string .= format_keywords($dbh,\@columns,$table)." ";
		}
	}
	my @columns = ();
	push(@columns,$current_id);
	for(my $i=2;$i<$count;$i++)
	{
		@tmp = split('=',$pairs[$i]);
		push(@columns,$tmp[1]);		
	}
	$keyword_string .= format_keywords($dbh,\@columns,$table)." ";		
	debug_log("editable_table_insert.pl, keyword_string - ".$keyword_string);

	check_data($dbh,$table,undef,$keyword_string,$stanza_name,"editable_add");
}

# end of process input string

# the following 5 tables have a foreign key - clusterID - to TLLR_CFGMCluster.
# so, if there is no record in TLLR_CFGMCluster, add a new record first.
if($table eq "TLLR_CFGMClusterClasses" || $table eq "TLLR_CFGMClusterGroups" || $table eq "TLLR_CFGMClusterUsers"
   || $table eq "TLLR_CFGMClusterInboundHosts" || $table eq "TLLR_CFGMClusterOutboundHosts")
{
	my $cluster_id;
	for(my $i=1;$i<$count;$i++)
	{
		@tmp = split('=',$pairs[$i]);
		if($tmp[0] eq "clusterID")
		{
			$cluster_id = $tmp[1];
		}
	}

	my $sql_str="select clusterID from TLLR_CFGMCluster where clusterID=".$cluster_id;
	my $sth=$dbh->prepare($sql_str);
	if(!defined($sth->execute()))
	{
		error_and_exit($dbh->errstr);
	}

	my($id);
	$sth->bind_columns(undef,\$id);
	my $existed = 0;
	if($sth->fetch())
	{
		$existed = 1;
	}
	
	if($existed == 0)
	{
		my $statement = "insert into TLLR_CFGMCluster set clusterID=".$cluster_id;
		debug_log("editable_table_insert.pl, add to TLLR_CFGMCluster firstly, statement: ".$statement);
		if(!defined($dbh->do($statement)))
		{
			error_and_exit($dbh->errstr);
		}		
	}
}

# add a new record
if(!defined($dbh->do($insert_str)))
{
	error_and_exit($dbh->errstr);
}

# If a new record is added to TLLR_CFGMachineGroupMachineList, we should support the regular expression, and insert each machine into TLL_Nodelist.
# editable_table_insert.pl?table=TLLR_CFGMachineGroupMachineList&mgID=2&machines=ab
if($table eq "TLLR_CFGMachineGroupMachineList")
{
	my $machines;
	for(my $i=1;$i<$count;$i++)
	{
		my @tmp = split('=',$pairs[$i]);
		if($tmp[0] eq "machines")
		{
			$machines = $tmp[1];
		}
	}
	debug_log("haha-------------------------------------------------------------".$machines);
	if(defined($machines))
	{
		my %include_machines = ();
		my %exclude_machines = ();
		my @strs = split(',',$machines);
		foreach my $a (@strs)
		{
			debug_log("haha+++++:".$a);
			$_ = $a;
			if(/^-(.+)/)
			{
				$exclude_machines{$1} = 1;
			}
			elsif(/^(\w+)\+(\d+)$/)
			{
				# pccluster01+999
				my $tmp1 = $1;
				my $end = $2;
				if($tmp1 =~ /^(\w*[a-zA-Z]+)*(\d+)$/)
				{
					my $prefix = $1;
					my $begin = $2;
					my @arr = &expand_to_array($begin,$end+$begin+1);
					foreach my $e (@arr)
					{
						my $m = $prefix.$e;
						$include_machines{$m} = 1;
					}
				}
				else
				{
					debug_log("error:".$_);
				}
			}
			elsif(/^(\w+)-(\d+)$/)
			{
				# pccluster1-1000
				my $tmp1 = $1;
				my $end = $2;
				if($tmp1 =~ /^(\w*[a-zA-Z]+)*(\d+)$/)
				{
					my $prefix = $1;
					my $begin = $2;
					my @arr = &expand_to_array($begin,$end+1);
					foreach my $e (@arr)
					{
						my $m = $prefix.$e;
						$include_machines{$m} = 1;
					}
				}
				else
				{
					debug_log("error:".$_);
				}
			}
			elsif(/^(\w*)\[(\d+)-(\d+)\](\w*)$/)
			{
				# pccluster[1-1000]
				my @arr = &expand_to_array($2,$3+1);
				foreach my $e (@arr)
				{
					my $m = $1.$e.$4;
					$include_machines{$m} = 1;
				}
			}
			elsif(/^(\w*)\[(\d+):(\d+)\](\w*)$/)
			{
				# c197blade1b[01:19]
				my @arr = &expand_to_array($2,$3+1);
				foreach my $e (@arr)
				{
					my $m = $1.$e.$4;
					$include_machines{$m} = 1;
				}				
			}
			elsif(/^(\w+)-(\w+)$/)
			{
				# x330n01-x330n123
				my $tmp1 = $1;
				my $tmp2 = $2;
				if($tmp1 =~ /^(\w*[a-zA-Z]+)*(\d+)$/)
				{
					my $prefix1 = $1;
					my $begin = $2;
					if($tmp2 =~ /^(\w*[a-zA-Z]+)*(\d+)$/)
					{
						my $prefix2 = $1;
						my $end = $2;
						if($prefix1 eq $prefix2)
						{
							my @arr = &expand_to_array($begin,$end+1);
							foreach my $e (@arr)
							{
								my $m = $prefix1.$e;
								$include_machines{$m} = 1;
							}
						}
						else
						{
							debug_log("error: different prefix: ".$_);
						}
					}
					else
					{
						debug_log("error:".$_);
					}
				}
				else
				{
					debug_log("error:".$_);
				}
			}
			else
			{
				$include_machines{$_} = 1;
			}
		}

		foreach my $m (keys %include_machines)
		{
			if(!defined($exclude_machines{$m}))
			{
				debug_log($m);
				$insert_str = "insert into TLL_Nodelist set hostname='".$m."', clusterID=1";
				if(!defined($dbh->do($insert_str)))
				{
					error_and_exit($dbh->errstr);
				}
			}
		}
	}
	else
	{
		error_and_exit("machines not defined.");
	}
	debug_log("haha-------------------------------------------------------------");
}

$dbh->disconnect();

print LLUtil::OUTPUT_HEADER."OK";

sub expand_to_array
{
	my $begin = $_[0];
	my $end = $_[1];
	my @rs = ();
	my $len = rindex $2."\$", "\$";
#	print "len:".$len."\n";
	for(my $i=$begin;$i<$end;$i++)
	{
		my $tmp = sprintf "%.".$len."d",$i;
		push(@rs,$tmp);
	}
	@rs;
}

==> LL_utils-5.1.0.11-x86_64-Linux-RHEL6.tar.gz: var/www/ll/cgi-bin/editable_table_insert.pl <==
#!/usr/bin/perl

# static char sccsid[] = "@(#)14 1.13 src/ll/config_editor/cgi-bin/editable_table_insert.pl, ll.config, ll_rrnep2, rrnep21206a 7/15/11 01:44:12\n";

use strict;
use lib "/var/www/ll/cgi-bin";
use LLUtil;
use DBI;

# table=TLLR_CFGMachineClass&nodeID=2&class_name=c&class_count=4&type=all_value&name=default&class_name=a&class_count=2&class_name=b&class_count=3
# table=TLLR_CFGMachineClass&nodeID=21&class_name=g&class_count=8&type=all_value&name=faaffa
my $query = $ENV{QUERY_STRING};
$query = unescape($query);
debug_log("query: ".$query);

# connect to database
my $dbh = get_dbh(0);
if(!defined($dbh))
{
	error_and_exit(DBI->errstr);
}

# beginning of process input string

my @pairs = split('&',$query);
my @tmp = split('=',$pairs[0]);
my $table = $tmp[1];
@tmp = split('=',$pairs[1]);
my $current_id = $tmp[1];
my $kw_name = "";

# format the insert SQL string
#    due to DB2 limitations format is table (cols) values (vals)
my $insert_cols="";
my $insert_vals="";
my $count = 1;
my $n = scalar(@pairs);
for(my $i=1;$i<$n;$i++)
{
	if($pairs[$i] eq "type=all_value")
	{
		last;
	}
	
	@tmp = split('=',$pairs[$i]);
	$insert_cols .= " ".$tmp[0].",";
	$insert_vals .= " '".$tmp[1]."',";
	$count++;
	
	if (($i == 2) && (($table eq "TLLS_CFGClassLimits") || $table eq "TLLR_CFGFileSysMonThresholds"))
	{
		# When insert some one keyword for table TLLS_CFGClassLimits or TLLR_CFGFileSysMonThresholds,
		# Only need to check the newly input keyword-value pair.
		# Extract the keyword name here to be used later in check_data() subroutine.
		$kw_name = $tmp[1];	
		debug_log("kw_name: ".$kw_name);
	}
}
$insert_cols =~ s/,$//g;
$insert_vals =~ s/,$//g;
my $insert_str="insert into ".$table." (".$insert_cols.") values(".$insert_vals.")";
debug_log($insert_str);
debug_log("count: ".$count);

# data checking (make use of check utility code)
{
	# get stanza name
	my $index = $count + 1;
	@tmp = split('=',$pairs[$index]);
	my $stanza_name = $tmp[1];

	# format the <keyword>,<value> pair for checking
	my $keyword_string = "";
	if(($index + 1 < $n) && ($table ne "TLLS_CFGClassLimits") && ($table ne "TLLR_CFGFileSysMonThresholds"))
	{
		# for keywords from table TLLS_CFGClassLimits or TLLR_CFGFileSysMonThresholds,
		# the existed <keyword>,<value> pairs are not necessary for checking.
		my $column_number = $count - 2;
		my $existed_rows = ($n - $index - 1) / $column_number;
		debug_log("log: n=".$n.", index=".$index.", column_number=".$column_number.", existed_rows=".$existed_rows);


		for(my $i=0;$i<$existed_rows;$i++)
		{
			my @columns = ();
			push(@columns,$current_id);
			for(my $j=0;$j<$column_number;$j++)
			{
				@tmp = split('=',$pairs[$index + 1 + $i * $column_number + $j]);
				push(@columns,$tmp[1]);				
			}
			$keyword_string .= format_keywords($dbh,\@columns,$table)." ";
		}
	}
	my @columns = ();
	my @user_keywords = ();
	push(@columns,$current_id);
	push(@user_keywords,$current_id);
	for(my $i=2;$i<$count;$i++)
	{
		@tmp = split('=',$pairs[$i]);
		push(@columns,$tmp[1]);		
		push(@user_keywords,$tmp[0]);	
	}
	$keyword_string .= format_keywords($dbh,\@columns,$table)." ";		
	debug_log("editable_table_insert.pl, keyword_string - ".$keyword_string);

	if (($table eq "TLLS_CFGClassLimits") || ($table eq "TLLR_CFGFileSysMonThresholds"))
	{
		# For keywords from table TLLS_CFGClassLimits or TLLR_CFGFileSysMonThresholds,
		# Only need to check the newly input keyword-value pair.
		check_data($dbh,$table,$kw_name,$keyword_string,$stanza_name,"editable_add");
	}elsif ($table eq "TLLS_CFGClassUser"){
		# For each keyword-value pair in user substanza in class stanza, check the validity
		for(my $i=2;$i<$count-1;$i++)
		{
			check_data($dbh,$table,$user_keywords[$i],$columns[$i],$stanza_name,"editable_add");	
		}
	}
	else
	{
		check_data($dbh,$table,undef,$keyword_string,$stanza_name,"editable_add");
	}
}

# end of process input string

# the following 5 tables have a foreign key - clusterID - to TLLR_CFGMCluster.
# so, if there is no record in TLLR_CFGMCluster, add a new record first.
if($table eq "TLLR_CFGMClusterClasses" || $table eq "TLLR_CFGMClusterGroups" || $table eq "TLLR_CFGMClusterUsers"
   || $table eq "TLLR_CFGMClusterInboundHosts" || $table eq "TLLR_CFGMClusterOutboundHosts")
{
	my $cluster_id;
	for(my $i=1;$i<$count;$i++)
	{
		@tmp = split('=',$pairs[$i]);
		if(lc($tmp[0]) eq lc("clusterID"))
		{
			$cluster_id = $tmp[1];
		}
	}

	my $sql_str="select clusterID from TLLR_CFGMCluster where clusterID=".$cluster_id;
	my $sth=$dbh->prepare($sql_str);
	if(!defined($sth->execute()))
	{
		error_and_exit($dbh->errstr);
	}

	my($id);
	$sth->bind_columns(undef,\$id);
	my $existed = 0;
	if($sth->fetch())
	{
		$existed = 1;
	}
	
	if($existed == 0)
	{
		my $statement = "insert into TLLR_CFGMCluster (clusterID) values(".$cluster_id.")";
		debug_log("editable_table_insert.pl, add to TLLR_CFGMCluster firstly, statement: ".$statement);
		if(!defined($dbh->do($statement)))
		{
			error_and_exit($dbh->errstr);
		}		
	}
}

# add a new record
if(!defined($dbh->do($insert_str)))
{
	error_and_exit($dbh->errstr);
}

# If a new record is added to TLLR_CFGMachineGroupMachineList, we should support the regular expression, and insert each machine into TLL_Nodelist.
# editable_table_insert.pl?table=TLLR_CFGMachineGroupMachineList&mgID=2&machines=ab
if($table eq "TLLR_CFGMachineGroupMachineList")
{
	my $machines;
	for(my $i=1;$i<$count;$i++)
	{
		my @tmp = split('=',$pairs[$i]);
		if(lc($tmp[0]) eq "machines")
		{
			$machines = $tmp[1];
		}
	}
	debug_log("haha-------------------------------------------------------------".$machines);
	if(defined($machines))
	{
		my %include_machines = ();
		my %exclude_machines = ();
		my @strs = split(',',$machines);
		foreach my $a (@strs)
		{
			debug_log("haha+++++:".$a);
			$_ = $a;
			if(/^-(.+)/)
			{
				$exclude_machines{$1} = 1;
			}
			elsif(/^(\w+)\+(\d+)$/)
			{
				# pccluster01+999
				my $tmp1 = $1;
				my $end = $2;
				if($tmp1 =~ /^(\w*[a-zA-Z]+)*(\d+)$/)
				{
					my $prefix = $1;
					my $begin = $2;
					my @arr = &expand_to_array($begin,$end+$begin+1);
					foreach my $e (@arr)
					{
						my $m = $prefix.$e;
						$include_machines{$m} = 1;
					}
				}
				else
				{
					debug_log("error:".$_);
				}
			}
			elsif(/^(\w+)-(\d+)$/)
			{
				# pccluster1-1000
				my $tmp1 = $1;
				my $end = $2;
				if($tmp1 =~ /^(\w*[a-zA-Z]+)*(\d+)$/)
				{
					my $prefix = $1;
					my $begin = $2;
					my @arr = &expand_to_array($begin,$end+1);
					foreach my $e (@arr)
					{
						my $m = $prefix.$e;
						$include_machines{$m} = 1;
					}
				}
				else
				{
					debug_log("error:".$_);
				}
			}
			elsif(/^(\w*)\[(\d+)-(\d+)\](\w*)$/)
			{
				# pccluster[1-1000]
				my @arr = &expand_to_array($2,$3+1);
				foreach my $e (@arr)
				{
					my $m = $1.$e.$4;
					$include_machines{$m} = 1;
				}
			}
			elsif(/^(\w*)\[(\d+):(\d+)\](\w*)$/)
			{
				# c197blade1b[01:19]
				my @arr = &expand_to_array($2,$3+1);
				foreach my $e (@arr)
				{
					my $m = $1.$e.$4;
					$include_machines{$m} = 1;
				}				
			}
			elsif(/^(\w+)-(\w+)$/)
			{
				# x330n01-x330n123
				my $tmp1 = $1;
				my $tmp2 = $2;
				if($tmp1 =~ /^(\w*[a-zA-Z]+)*(\d+)$/)
				{
					my $prefix1 = $1;
					my $begin = $2;
					if($tmp2 =~ /^(\w*[a-zA-Z]+)*(\d+)$/)
					{
						my $prefix2 = $1;
						my $end = $2;
						if($prefix1 eq $prefix2)
						{
							my @arr = &expand_to_array($begin,$end+1);
							foreach my $e (@arr)
							{
								my $m = $prefix1.$e;
								$include_machines{$m} = 1;
							}
						}
						else
						{
							debug_log("error: different prefix: ".$_);
						}
					}
					else
					{
						debug_log("error:".$_);
					}
				}
				else
				{
					debug_log("error:".$_);
				}
			}
			else
			{
				$include_machines{$_} = 1;
			}
		}

		foreach my $m (keys %include_machines)
		{
			if(!defined($exclude_machines{$m}))
			{
				debug_log($m);
				$insert_str = "insert into TLL_Nodelist (hostname,clusterID) values('".$m."', '1')";
				if(!defined($dbh->do($insert_str)))
				{
					error_and_exit($dbh->errstr);
				}
			}
		}
	}
	else
	{
		error_and_exit("machines not defined.");
	}
	debug_log("haha-------------------------------------------------------------");
}

$dbh->disconnect();

print LLUtil::OUTPUT_HEADER."OK";

sub expand_to_array
{
	my $begin = $_[0];
	my $end = $_[1];
	my @rs = ();
	my $len = rindex $2."\$", "\$";
#	print "len:".$len."\n";
	for(my $i=$begin;$i<$end;$i++)
	{
		my $tmp = sprintf "%.".$len."d",$i;
		push(@rs,$tmp);
	}
	@rs;
}
